(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common/http'), require('apollo-link'), require('apollo-link-batch'), require('graphql/language/printer'), require('apollo-angular-link-http-common')) :
    typeof define === 'function' && define.amd ? define('apollo-angular-link-http-batch', ['exports', '@angular/core', '@angular/common/http', 'apollo-link', 'apollo-link-batch', 'graphql/language/printer', 'apollo-angular-link-http-common'], factory) :
    (global = global || self, factory(global['apollo-angular-link-http-batch'] = {}, global.ng.core, global.ng.common.http, global.apolloLink.core, global.apolloLink.batch, global.printer, global.ng.apolloLink.httpCommon));
}(this, function (exports, core, http, apolloLink, apolloLinkBatch, printer, apolloAngularLinkHttpCommon) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    /* global Reflect, Promise */

    var extendStatics = function(d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };

    function __extends(d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }

    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }

    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(metadataKey, metadataValue);
    }

    var defaults = {
        batchInterval: 10,
        batchMax: 10,
        uri: 'graphql',
        method: 'POST',
    };
    var HttpBatchLinkHandler = /** @class */ (function (_super) {
        __extends(HttpBatchLinkHandler, _super);
        function HttpBatchLinkHandler(httpClient, options) {
            var _this = _super.call(this) || this;
            _this.httpClient = httpClient;
            _this.options = options;
            _this.batchInterval = options.batchInterval || defaults.batchInterval;
            _this.batchMax = options.batchMax || defaults.batchMax;
            var batchHandler = function (operations) {
                return new apolloLink.Observable(function (observer) {
                    var body = _this.createBody(operations);
                    var headers = _this.createHeaders(operations);
                    var _a = _this.createOptions(operations), method = _a.method, uri = _a.uri, withCredentials = _a.withCredentials;
                    if (typeof uri === 'function') {
                        throw new Error("Option 'uri' is a function, should be a string");
                    }
                    var req = {
                        method: method,
                        url: uri,
                        body: body,
                        options: {
                            withCredentials: withCredentials,
                            headers: headers,
                        },
                    };
                    var sub = apolloAngularLinkHttpCommon.fetch(req, _this.httpClient, function () {
                        throw new Error('File upload is not available when combined with Batching');
                    }).subscribe({
                        next: function (result) { return observer.next(result.body); },
                        error: function (err) { return observer.error(err); },
                        complete: function () { return observer.complete(); },
                    });
                    return function () {
                        if (!sub.closed) {
                            sub.unsubscribe();
                        }
                    };
                });
            };
            var batchKey = options.batchKey ||
                (function (operation) {
                    return _this.createBatchKey(operation);
                });
            _this.batcher = new apolloLinkBatch.BatchLink({
                batchInterval: _this.batchInterval,
                batchMax: _this.batchMax,
                batchKey: batchKey,
                batchHandler: batchHandler,
            });
            return _this;
        }
        HttpBatchLinkHandler.prototype.createOptions = function (operations) {
            var context = operations[0].getContext();
            return {
                method: apolloAngularLinkHttpCommon.prioritize(context.method, this.options.method, defaults.method),
                uri: apolloAngularLinkHttpCommon.prioritize(context.uri, this.options.uri, defaults.uri),
                withCredentials: apolloAngularLinkHttpCommon.prioritize(context.withCredentials, this.options.withCredentials),
            };
        };
        HttpBatchLinkHandler.prototype.createBody = function (operations) {
            var _this = this;
            return operations.map(function (operation) {
                var includeExtensions = apolloAngularLinkHttpCommon.prioritize(operation.getContext().includeExtensions, _this.options.includeExtensions, false);
                var includeQuery = apolloAngularLinkHttpCommon.prioritize(operation.getContext().includeQuery, _this.options.includeQuery, true);
                var body = {
                    operationName: operation.operationName,
                    variables: operation.variables,
                };
                if (includeExtensions) {
                    body.extensions = operation.extensions;
                }
                if (includeQuery) {
                    body.query = printer.print(operation.query);
                }
                return body;
            });
        };
        HttpBatchLinkHandler.prototype.createHeaders = function (operations) {
            return operations.reduce(function (headers, operation) {
                return apolloAngularLinkHttpCommon.mergeHeaders(headers, operation.getContext().headers);
            }, this.options.headers);
        };
        HttpBatchLinkHandler.prototype.createBatchKey = function (operation) {
            var context = operation.getContext();
            if (context.skipBatching) {
                return Math.random()
                    .toString(36)
                    .substr(2, 9);
            }
            var headers = context.headers &&
                context.headers.keys().map(function (k) { return context.headers.get(k); });
            var opts = JSON.stringify({
                includeQuery: context.includeQuery,
                includeExtensions: context.includeExtensions,
                headers: headers,
            });
            return apolloAngularLinkHttpCommon.prioritize(context.uri, this.options.uri) + opts;
        };
        HttpBatchLinkHandler.prototype.request = function (op) {
            return this.batcher.request(op);
        };
        return HttpBatchLinkHandler;
    }(apolloLink.ApolloLink));
    var HttpBatchLink = /** @class */ (function () {
        function HttpBatchLink(httpClient) {
            this.httpClient = httpClient;
        }
        HttpBatchLink.prototype.create = function (options) {
            return new HttpBatchLinkHandler(this.httpClient, options);
        };
        HttpBatchLink = __decorate([
            core.Injectable(),
            __metadata("design:paramtypes", [http.HttpClient])
        ], HttpBatchLink);
        return HttpBatchLink;
    }());

    var PROVIDERS = [HttpBatchLink];
    var HttpBatchLinkModule = /** @class */ (function () {
        function HttpBatchLinkModule() {
        }
        HttpBatchLinkModule = __decorate([
            core.NgModule({
                providers: PROVIDERS,
            })
        ], HttpBatchLinkModule);
        return HttpBatchLinkModule;
    }());

    exports.HttpBatchLink = HttpBatchLink;
    exports.HttpBatchLinkHandler = HttpBatchLinkHandler;
    exports.HttpBatchLinkModule = HttpBatchLinkModule;
    exports.PROVIDERS = PROVIDERS;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=ng.apolloLink.httpBatch.umd.js.map
